#!/usr/bin/python
import sys

'''
    Gauss-Seidel-Verfahren zur Loesung eines diagonaldominanten linearen
    Gleichungssystems A x = b bestehend aud n Gleichungen mit n Unbekannten;
    vorausgesetzt wird, dass alle Diagonalelemente von 0 verschieden sind
'''

def gsiteration(a, x, b, n):
# Unterprogramm, das eine Gauss-Seidel-Iteration durchfuehrt
   for i in range (0,n):
      tmp = b[i]
      for j in range (0,n):
         if j!=i :
            tmp = tmp - a[i][j]*x[j]
      x[i] = tmp/a[i][i]

def test_zsk(a, x, b, n):
# Unterprogramm, das testet, ob das starke Zeilensummenkriterium erfuellt ist,
# und den Faktor, um den der Fehler pro Iterationsschritt mindestens reduziert
# wird, ausgibt.
   print
   print "ANALYSE ZEILENSUMMENKRITERIUM:"
   if n==1:
      print "Das GLS besteht nur aus einer Gleichung und liefert daher "
      print "(bis auf etwaige Rundungsfehler) die Loesung des GLS in "
      print "einem Iterationsschritt."
   else:
      factor = 0.
      for i in range (0,n):
         tmp = 0.
         for j in range (0,n):
            if j!=i :
               tmp = tmp + abs(a[i][j])
         factor = max(factor, tmp/abs(a[i][i]))
      if factor < 1.:
         print "Das starke Zeilensummenkriterium ist bei dem gegebenen GLS erfuellt."
         print "Pro Iterationsschritt wird der Fehler mindestens um den Faktor ",factor," reduziert."
      elif factor ==1.:
         print "Es ist nur das schwache Zeilensummenkriterium erfuellt."
         print "Konvergenz aufgrund des starken Zeilensummenkriteriums ist nicht gesichert."
      else:
         print "Die Matrix ist nicht diagonaldominant."
         print "Es ist nicht sicher, ob das Gauss-Seidel-Verfahren gegen die Loesung des GLS konvergiert."
   print
   return

def resicalc(a, x, b, n):
# Unterprogramm zur Berechnung des maximalen Residuums/Defekts
   locresi=range(n)
   for i in range (0,n):
      locresi[i] = b[i]
      for j in range (0,n):
         locresi[i] = locresi[i]-a[i][j]*x[j]
      locresi[i]=abs(locresi[i])
   maxresi = max(locresi)
   return maxresi

#################
# HAUPTPROGRAMM #
#################

# Zentrale Parameter:
#  n     : Zahl der Unbekannten
#  niter  : Maximalzahl der durchzufuehrenden Iterationen
#  epsilon: Genauigkeit, mit der das GLS geloest werden soll;
#           die Gauss-Seidel-Iteration soll abbrechen,
#           wenn der Defekt/das Residuum kleiner
#           oder gleich epsilon geworden ist.


print
print 'Gauss-Seidel-Programm zur Loesung eines diagonaldominanten linearen Gleichungssystems'
print 'Das Programm endet spaetestens, wenn eine vorgegebene maximale Anzahl von'
print 'Iterationsschritten durchgefuehrt worden ist.'
print 'Das Programm endet vorher, wenn das (maximale) Residuum unter die Schranke epsilon gesunken ist.'
print
n = input('Zahl der Unbekannten des linearen Gleichungssystems: ')
niter = input('Maximale Anzahl von Iterationsschritten: ')
epsilon = float(input('Schranke fuer das Residuum'))
print
print 'Anzahl der Gleichungen: ', n 
print 'Schranke fuer Residuum: ', epsilon
print 'Maximale Anzahl der Iterationen: ', niter
print
# Anlegen der n-dimensionalen Vektoren x und b und der (n x n) - Matrix a
a=range(n)
b=range(n)
x=range(n)
# Eingabe von a und b
print 'gib die Matrix-Koeffizienten der einzelnen Gleichungen durch Kommata getrennt ein; '
print 'Beispiel fuer n=4: 11.3, 2.7, 3, 4.7'
for i in range(1,n+1):
   txt= 'Koeffizienten der '+str(i)+'-ten Gleichung: '
   if n==1:
      a[0]=range(n)
      a[0][0]=float(input(txt))
   else:
      a[i-1]=input(txt)
print 'gib alle Werte der rechten Seite des linearen Gleichungssystems durch Kommate getrennt ein; '
print 'Beispiel: 37.4, 39.8, 501, 701.1' 
if n==1:
   b[0]=float(input('rechte Seite des Gleichungssystems: '))
else:
   b=input('rechte Seite des Gleichungssystems: ')

# Setzen einer Startnaeherung fuer x
for i in range (0,n):
   x[i]=0.

# Test Zeilensummenkriterium
test_zsk(a,x,b,n)
# Gauss-Seidel-Iterationen mit Abbruchkriterium
for iter in range (1,niter+1):
   gsiteration(a,x,b,n)
   maxresi=resicalc(a,x,b,n)
   print 'Iteration Nr. ',iter,'; maximales Residuum =', maxresi
   if maxresi <= epsilon:
      print 'Das maximale Residuum erreicht die vorgegebene Schranke ',epsilon
      print 'nach ',iter,' Iterationsschritten'
      break
   
# Ausgabe der Loesung
print 'Die berechnete Loesung lautet x = ', x
sys.stdin.readline()
